## How we the creators expect MiniIDE to behave in a few different scenarios

1. Typing in an expression involving constants into either the console (bottom
   line on screen) or editor evaluates it and prints the result in the console
   (above the bottom line on screen).

    ```
    > 1+1
    2
    ```

    In editor:
    ```
    1+1
    ```
    and hit 'Run'

2. Typing in graphics involving constants into either the console or editor
   shows them on screen.

    ```
    > love.graphics.line(100,100, 200,300)
    ```

    You should see a line on the screen, potentially drawing over text in the
    console.

3. Typing in an expression involving variables into either the console or
   editor works as usual.

    ```
    > x = 1
    > x+1
    2
    ```

    In editor, type:
    ```
    x = 1
    ```
    and hit 'Run'

    Then clear editor and type:
    ```
    x+1
    ```
    and hit 'Run' again.

    (You can't put both statements into the editor at once. That would be more
    like typing `x = 1; x+1` into the console or any Lua prompt. Which won't
    work on either.)

4. Drawing graphics involving variables will work in either the console or
   editor.
    ```
    x = 100
    love.graphics.line(x,x, 200,300)
    ```

    The console requires blocks to all be on a single line, though:
    ```
    > function foo()
    ```
    If you hit enter now you'll see an error. You _can_ define functions if
    they're all on one line.
    ```
    > function foo() return 42 end
    ```

    You're probably better off putting function definitions in the editor.

5. However, graphics only work when they fit a specific pattern. This doesn't
   work (known bug):
    ```
    lg = love.graphics
    lg.line(100,100, 200,300)
    ```

6. For more complex graphics, override `love.draw`:
    ```
    lg = love.graphics

    function love.draw()
        lg.line(100,100, 200,300)
    end
    ```

    However, once you hit 'Run' you're running a whole new app, and you'll
    have no way to go back to MiniIDE. (intended behavior)

7. To return to MiniIDE, override some event to `escape()`:
    ```
    function love.draw()
        love.graphics.line(100,100, 200,300)
    end
    function love.keypressed(key)
        escape()
    end
    ```

    Now press the 'Stop Code' button. It will insert some code into the editor
    so it looks like this:
    ```
    function love.draw()
        love.graphics.line(100,100, 200,300)
    end
    function love.keypressed(key)
        escape()
    end
    function escape()
        error('StopScript')
    end
    ```

    Now press 'Run' to see a new app draw a line across the whole screen.
    Press any key to return to MiniIDE.

8. You can also create an app that includes MiniIDE, like this:
    ```
    lg = love.graphics

    if old_love_draw == nil then
        old_love_draw = love.draw
    end

    function love.draw()
        old_love_draw()
        lg.line(100,100, 200,300)
    end
    ```

    This will look just like scenario 2 above, but it has more room to grow.
    You can take over other event handlers from MiniIDE like `love.keypressed`
    or `love.mousepressed`, and you can add rules before or after them. Any
    errors will put you back in MiniIDE without your app additions. For
    example, if you edit it to look like this:

    ```
    lg = love.graphics

    if old_love_draw == nil then
        old_love_draw = love.draw
    end

    function love.draw()
        old_love_draw()
        lg.line(100,100, 200,300)
    end

    function love.keypressed()
        print(y+1)  -- error: y is nil, and you can't add 1 to nil
    end
    ```

    Hit 'Run', then press a key. The key will have no effect, and you'll see
    an error like this in the log:
    ```
    Error: [string "lg = love.graphics..."]:13: attempt to perform arithmetic on global 'y' (a nil value)
    ```

    However, future keypresses (without hitting 'Run' again) will work. Your
    app has been 'deregistered', and things have gone back to pristine
    MiniIDE. Now you can fix the error and try hitting 'Run' again.
